# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (c) 2012, Peter Levi <peterlevi@peterlevi.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE
import subprocess
import re
from variety.plugins.IQuoteSource import IQuoteSource
from locale import gettext as _
import logging

logger = logging.getLogger("variety")


class FortunesSource(IQuoteSource):
    def __init__(self):
        super(IQuoteSource, self).__init__()
        self.quotes = []

    @classmethod
    def get_info(cls):
        return {
            "name": "Fortunes",
            "description": _("Displays quotes, defined in fortunes command."),
            "author": "Alex Vergara Gil",
            "version": "0.1"
        }

    def supports_search(self):
        return True

    def activate(self):
        if self.active:
            return

        super(FortunesSource, self).activate()

        self.load()

    def deactivate(self):
        self.quotes = []

    def load(self):
        self.quotes = []
        try:
            logger.info("Loading fortunes quotes")
            s = subprocess.Popen(
                "fortune",
                shell=True,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True
            )
            r = ''
            while True:
                l = s.stdout.readline()
                if not l:
                    break
                r += l
            logger.info("Loading quote %s" % r)
            for q in re.split(r'(^\.$|^%$)', r, flags=re.MULTILINE):
                try:
                    if q.strip():
                        parts = q.split('-- ')
                        quote = ' '.join(parts[0].split())
                        if quote[0] == quote[-1] == '"':
                            quote = u"\u201C%s\u201D" % quote[1:-1]
                        author = parts[1].strip() if len(parts) > 1 else None
                        self.quotes.append({"quote": quote, "author": author})
                except Exception:
                    logger.debug('Could not process fortune quote %s' % q)
        except Exception:
            logger.exception("Could not load fortunes quotes")

    def get_random(self):
        self.load()
        return self.quotes

    def get_for_author(self, author):
        return [q for q in self.quotes if q["author"] and q["author"].lower().find(author.lower()) >= 0]

    def get_for_keyword(self, keyword):
        return self.get_for_author(keyword) + \
               [q for q in self.quotes if q["quote"].lower().find(keyword.lower()) >= 0]